from .core import hints

Box = hints.Box
Contour = hints.Contour
Empty = hints.Empty
Linear = hints.Linear
Maybe = hints.Maybe
Mix = hints.Mix
Multipoint = hints.Multipoint
Multipolygon = hints.Multipolygon
Multisegment = hints.Multisegment
Point = hints.Point
Polygon = hints.Polygon
Scalar = hints.Scalar
Segment = hints.Segment
Shaped = hints.Shaped
Box.__module__ = __name__
Contour.__module__ = __name__
Empty.__module__ = __name__
Mix.__module__ = __name__
Multipoint.__module__ = __name__
Multipolygon.__module__ = __name__
Multisegment.__module__ = __name__
Point.__module__ = __name__
Polygon.__module__ = __name__
Segment.__module__ = __name__
